=== modified file 'INSTALL'
--- INSTALL	2011-03-27 10:06:36 +0000
+++ INSTALL	2011-04-06 10:03:08 +0000
@@ -104,6 +104,7 @@
   changephoto: must run as root
   deletephoto: must run as root
   resizeavatar: must run as root
+  exportaccount: must run as root
 
 There are python scripts under libravatar/ for all of these functions and
 workers can be setup like this:

=== modified file 'config/www.apache2.conf'
--- config/www.apache2.conf	2011-01-30 05:28:15 +0000
+++ config/www.apache2.conf	2011-04-06 10:03:08 +0000
@@ -39,11 +39,15 @@
     <Directory /var/lib/libravatar/user>
         Allow from all
     </Directory>
+    <Directory /var/lib/libravatar/export>
+        Allow from all
+    </Directory>
 
     Alias /favicon.ico /usr/share/libravatar/libravatar/favicon.ico
     Alias /robots.txt /usr/share/libravatar/libravatar/robots.txt
     Alias /uploaded /var/lib/libravatar/uploaded
     Alias /user /var/lib/libravatar/user
+    Alias /export /var/lib/libravatar/export
 
     <Location /uploaded>
         Header set Cache-Control "public"
@@ -54,6 +58,12 @@
         ErrorDocument 404 http://__CDNSERVERNAME__/img/missing.png
     </Location>
 
+    <Location /export>
+        # Force browsers to download this instead of serving it
+        ForceType application/octet-stream
+        Header set Content-Disposition attachment
+    </Location>
+
     # Clients should be using cdn.libravatar.org/avatar/
     RewriteEngine on
     RewriteRule ^/avatar/ - [last]

=== added file 'debian/libravatar-www.exportaccount.init'
--- debian/libravatar-www.exportaccount.init	1970-01-01 00:00:00 +0000
+++ debian/libravatar-www.exportaccount.init	2011-04-06 10:03:08 +0000
@@ -0,0 +1,128 @@
+#!/bin/sh
+### BEGIN INIT INFO
+# Provides:          exportaccount
+# Required-Start:    $network $local_fs $remote_fs gearman-job-server
+# Required-Stop:     $remote_fs
+# Default-Start:     2 3 4 5
+# Default-Stop:      0 1 6
+# Short-Description: Gearman worker for exporting user data
+# Description:       Gearman worker for exporting user data
+### END INIT INFO
+
+# Author: Francois Marier <francois@debian.org>
+
+# PATH should only include /usr/* if it runs after the mountnfs.sh script
+PATH=/sbin:/usr/sbin:/bin:/usr/bin
+DESC="Libravatar exportaccount worker"
+NAME=exportaccount
+PIDDIR=/var/run/libravatar
+PIDFILE=$PIDDIR/$NAME.pid
+DAEMON=/usr/bin/gearman
+DAEMON_ARGS="-w -i $PIDFILE -f exportaccount /usr/share/libravatar/libravatar/exportaccount.py"
+SCRIPTNAME=/etc/init.d/$NAME
+
+# Exit if the package is not installed
+[ -x $DAEMON ] || exit 0
+
+# Read configuration variable file if it is present
+[ -r /etc/default/$NAME ] && . /etc/default/$NAME
+
+# Load the VERBOSE setting and other rcS variables
+. /lib/init/vars.sh
+
+# Define LSB log_* functions.
+# Depend on lsb-base (>= 3.0-6) to ensure that this file is present.
+. /lib/lsb/init-functions
+
+#
+# Function that starts the daemon/service
+#
+do_start()
+{
+	mkdir -p ${PIDDIR}
+	# Return
+	#   0 if daemon has been started
+	#   1 if daemon was already running
+	#   2 if daemon could not be started
+	start-stop-daemon --start --quiet --pidfile $PIDFILE --background --exec $DAEMON --test > /dev/null \
+		|| return 1
+	start-stop-daemon --start --quiet --pidfile $PIDFILE --background --exec $DAEMON -- \
+		$DAEMON_ARGS \
+		|| return 2
+}
+
+#
+# Function that stops the daemon/service
+#
+do_stop()
+{
+	# Return
+	#   0 if daemon has been stopped
+	#   1 if daemon was already stopped
+	#   2 if daemon could not be stopped
+	#   other if a failure occurred
+	start-stop-daemon --stop --quiet --retry=TERM/10/KILL/5 --pidfile $PIDFILE
+	RETVAL="$?"
+	[ "$RETVAL" = 2 ] && return 2
+	rm -f $PIDFILE
+	return "$RETVAL"
+}
+
+#
+# Function that sends a SIGHUP to the daemon/service
+#
+do_reload() {
+	#
+	# If the daemon can reload its configuration without
+	# restarting (for example, when it is sent a SIGHUP),
+	# then implement that here.
+	#
+	start-stop-daemon --stop --signal 1 --quiet --pidfile $PIDFILE --name $NAME
+	return 0
+}
+
+case "$1" in
+  start)
+    [ "$VERBOSE" != no ] && log_daemon_msg "Starting $DESC " "$NAME"
+    do_start
+    case "$?" in
+		0|1) [ "$VERBOSE" != no ] && log_end_msg 0 ;;
+		2) [ "$VERBOSE" != no ] && log_end_msg 1 ;;
+	esac
+  ;;
+  stop)
+	[ "$VERBOSE" != no ] && log_daemon_msg "Stopping $DESC" "$NAME"
+	do_stop
+	case "$?" in
+		0|1) [ "$VERBOSE" != no ] && log_end_msg 0 ;;
+		2) [ "$VERBOSE" != no ] && log_end_msg 1 ;;
+	esac
+	;;
+  status)
+       status_of_proc -p "$PIDFILE" "$DAEMON" "$NAME" && exit 0 || exit $?
+       ;;
+  restart|force-reload)
+	log_daemon_msg "Restarting $DESC" "$NAME"
+	do_stop
+	case "$?" in
+	  0|1)
+		do_start
+		case "$?" in
+			0) log_end_msg 0 ;;
+			1) log_end_msg 1 ;; # Old process is still running
+			*) log_end_msg 1 ;; # Failed to start
+		esac
+		;;
+	  *)
+	  	# Failed to stop
+		log_end_msg 1
+		;;
+	esac
+	;;
+  *)
+	echo "Usage: $SCRIPTNAME {start|stop|status|restart|force-reload}" >&2
+	exit 3
+	;;
+esac
+
+:

=== modified file 'debian/libravatar-www.postinst'
--- debian/libravatar-www.postinst	2011-04-02 08:58:06 +0000
+++ debian/libravatar-www.postinst	2011-04-06 10:03:08 +0000
@@ -32,6 +32,7 @@
 READY_FILES_ROOT=/var/lib/libravatar/ready
 UPLOADED_FILES_ROOT=/var/lib/libravatar/uploaded
 USER_FILES_ROOT=/var/lib/libravatar/user
+EXPORT_FILES_ROOT=/var/lib/libravatar/export
 APACHE_CONFIG=/etc/libravatar/www.apache2.conf
 
 case "$1" in
@@ -87,6 +88,7 @@
         set_config READY_FILES_ROOT "${READY_FILES_ROOT}/" ${SETTINGS_PY}
         set_config UPLOADED_FILES_ROOT "${UPLOADED_FILES_ROOT}/" ${SETTINGS_PY}
         set_config USER_FILES_ROOT "${USER_FILES_ROOT}/" ${SETTINGS_PY}
+        set_config EXPORT_FILES_ROOT "${EXPORT_FILES_ROOT}/" ${SETTINGS_PY}
 
         # Create a restricted user to crop/resize images with
         adduser --system --quiet --disabled-login --shell /bin/false --no-create-home --home ${READY_FILES_ROOT} --gecos 'Libravatar Image Processor,,,' libravatar-img || true
@@ -101,6 +103,9 @@
         mkdir -p ${USER_FILES_ROOT}
         chown -R root:root ${USER_FILES_ROOT}
 
+        mkdir -p ${EXPORT_FILES_ROOT}
+        chown -R root:root ${EXPORT_FILES_ROOT}
+
         touch /var/log/libravatar/workers.log
         chown libravatar-img /var/log/libravatar/workers.log
 

=== modified file 'debian/rules'
--- debian/rules	2011-03-27 10:06:36 +0000
+++ debian/rules	2011-04-06 10:03:08 +0000
@@ -13,3 +13,4 @@
 	dh_installinit --name=cropresize
 	dh_installinit --name=changephoto
 	dh_installinit --name=resizeavatar
+	dh_installinit --name=exportaccount

=== modified file 'libravatar/account/urls.py'
--- libravatar/account/urls.py	2011-03-21 11:09:56 +0000
+++ libravatar/account/urls.py	2011-04-06 10:03:08 +0000
@@ -37,6 +37,7 @@
     (r'^(?P<openid_id>\d+)/remove_unconfirmed_openid/$', 'libravatar.account.views.remove_unconfirmed_openid'),
 
     ('delete/$', 'libravatar.account.views.delete'),
+    ('export/$', 'libravatar.account.views.export'),
     ('new/$', 'libravatar.account.views.new'),
     ('password_reset/$', 'libravatar.account.views.password_reset'),
     ('password_reset_confirm/$', 'libravatar.account.views.password_reset_confirm'),

=== modified file 'libravatar/account/views.py'
--- libravatar/account/views.py	2011-04-02 10:08:03 +0000
+++ libravatar/account/views.py	2011-04-06 10:03:08 +0000
@@ -18,6 +18,9 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with Libravatar.  If not, see <http://www.gnu.org/licenses/>.
 
+from gearman import libgearman
+from hashlib import sha256
+import json
 from openid import oidutil
 from openid.consumer import consumer
 from StringIO import StringIO
@@ -545,12 +548,36 @@
     if request.method == 'POST':
         form = DeleteAccountForm(request.user, request.POST)
         if form.is_valid():
+            download_url = _perform_export(request.user, True)
             Photo.objects.delete_user_photos(request.user)
             request.user.delete() # cascading through unconfirmed and confirmed emails
             logout(request)
-            return render_to_response('account/delete_done.html', context_instance=RequestContext(request))
+            return render_to_response('account/delete_done.html', {'download_url': download_url},
+                                      context_instance=RequestContext(request))
     else:
         form = DeleteAccountForm(request.user)
 
     return render_to_response('account/delete.html', {'form' : form},
                               context_instance=RequestContext(request))
+
+def _perform_export(user, do_delete):
+    file_hash = sha256(user.username + user.password).hexdigest()
+    download_url = settings.EXPORT_FILES_URL + file_hash + '.xml'
+
+    gm_client = libgearman.Client()
+    for server in settings.GEARMAN_SERVERS:
+        gm_client.add_server(server)
+
+    workload = {'do_delete': do_delete, 'file_hash': file_hash}
+    gm_client.do_background('exportaccount', json.dumps(workload))
+    return download_url
+
+@csrf_protect
+@login_required
+def export(request):
+    if request.method == 'POST':
+        download_url = _perform_export(request.user, False)
+        return render_to_response('account/export_done.html', {'delete': delete, 'download_url': download_url},
+                                  context_instance=RequestContext(request))
+
+    return render_to_response('account/export.html', context_instance=RequestContext(request))

=== added file 'libravatar/exportaccount.py'
--- libravatar/exportaccount.py	1970-01-01 00:00:00 +0000
+++ libravatar/exportaccount.py	2011-04-06 10:03:08 +0000
@@ -0,0 +1,54 @@
+#!/usr/bin/python
+# Copyright (C) 2011  Francois Marier <francois@libravatar.org>
+#
+# This file is part of Libravatar
+# 
+# Libravatar is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published
+# by the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+# 
+# Libravatar is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+# 
+# You should have received a copy of the GNU Affero General Public License
+# along with Libravatar.  If not, see <http://www.gnu.org/licenses/>.
+
+import json
+import sys
+
+import settings # pylint: disable=W0403
+from utils import create_logger, is_hex # pylint: disable=W0403
+
+logger = create_logger('exportaccount')
+
+def main(argv=None):
+    if argv is None:
+        argv = sys.argv
+
+    gearman_workload = sys.stdin.read()
+    params = json.loads(gearman_workload)
+
+    do_delete = params['do_delete']
+    file_hash = params['file_hash']
+
+    # Validate inputs
+    if file_hash and not is_hex(file_hash):
+        logger.error('file_hash is not a hexadecimal value')
+        return 1
+
+    dest_filename = settings.EXPORT_FILES_ROOT + file_hash + '.xml'
+    destination = open(dest_filename, 'w')
+    destination.write('<?xml version="1.0"?>\n<libravatar-user>TODO</libravatar-user>')
+    destination.close()
+
+    if do_delete:
+        # TODO: kick off a photo deletion worker
+        pass
+
+    return 0
+
+if __name__ == "__main__":
+    sys.exit(main())

=== modified file 'libravatar/settings.py.example'
--- libravatar/settings.py.example	2011-04-02 08:58:06 +0000
+++ libravatar/settings.py.example	2011-04-06 10:03:08 +0000
@@ -129,6 +129,10 @@
 USER_FILES_ROOT = '%s/user/' % PROJECT_ROOT
 USER_FILES_URL = '%s/user/' % SITE_URL
 
+# Path and URL for user exports ready to be served
+EXPORT_FILES_ROOT = '%s/export/' % PROJECT_ROOT
+EXPORT_FILES_URL = '%s/export/' % SITE_URL
+
 # Path where avatars are stored
 AVATAR_ROOT = '%s/avatar/' % PROJECT_ROOT
 

=== modified file 'libravatar/templates/account/delete_done.html'
--- libravatar/templates/account/delete_done.html	2011-02-06 20:15:56 +0000
+++ libravatar/templates/account/delete_done.html	2011-04-06 10:03:08 +0000
@@ -5,9 +5,15 @@
 
 <h1>Account deleted</h1>
 
-<p>Your account has been permanently and completely deleted.</p>
-
-<p>Note that it may take a few days for your photos to disappear from all mirrors and a few weeks for your photos to disappear from web caches.</p>
+<p>Your account has been permanently and completely deleted. It may <b>take a few days</b> for your photos to disappear from all mirrors and a few weeks for your photos to disappear from web caches.</p>
+
+<p>In case you want to take your data with you, it's currenlty being exported and will be <b>ready in a few moments</b>:</p>
+
+<ul>
+  <li><a href="{{download_url}}">{{user.username}}.xml.gz</a></li>
+</ul>
+
+<p><i>This export will be deleted in a few hours.</i></p>
 
 <p><a href="{% url libravatar.public.views.home %}">Back to the homepage</a></p>
 

=== added file 'libravatar/templates/account/export.html'
--- libravatar/templates/account/export.html	1970-01-01 00:00:00 +0000
+++ libravatar/templates/account/export.html	2011-04-06 10:03:08 +0000
@@ -0,0 +1,17 @@
+{% extends 'base.html' %}
+{% block title %}Export your Libravatar data{% endblock title %}
+
+{% block content %}
+
+<h1>Export your data</h1>
+
+<p>Libravatar will now export all of your personal data to a compressed XML file.</p>
+
+<form action="{% url libravatar.account.views.export %}" method="post" name="export">{% csrf_token %}
+
+<p><input type="submit" value="Export">
+&nbsp;<a href="{% url libravatar.account.views.profile %}">Cancel</a></p>
+
+</form>
+
+{% endblock content %}

=== added file 'libravatar/templates/account/export_done.html'
--- libravatar/templates/account/export_done.html	1970-01-01 00:00:00 +0000
+++ libravatar/templates/account/export_done.html	2011-04-06 10:03:08 +0000
@@ -0,0 +1,18 @@
+{% extends 'base.html' %}
+{% block title %}Exporting Your Libravatar Data{% endblock title %}
+
+{% block content %}
+
+<h1>Export in progress</h1>
+
+<p>Libravatar is currently exporting your data. It <b>may take a few moments</b> for the file to be available:</p>
+
+<ul>
+  <li><a href="{{download_url}}">{{user.username}}.xml.gz</a></li>
+</ul>
+
+<p><i>This export will be deleted in a few hours.</i></p>
+
+<p><a href="{% url libravatar.account.views.profile %}">Back to your profile</a></p>
+
+{% endblock content %}

=== modified file 'libravatar/templates/account/profile.html'
--- libravatar/templates/account/profile.html	2011-04-02 03:39:16 +0000
+++ libravatar/templates/account/profile.html	2011-04-06 10:03:08 +0000
@@ -66,6 +66,8 @@
 
 <p><a href="{% url django.contrib.auth.views.password_change %}">Change your password</a></p>
 
+<p><a href="{% url libravatar.account.views.export %}">Export your data</a></p>
+
 <p><a href="{% url libravatar.account.views.delete %}">Permanently delete your account</a></p>
 
 {% endblock content %}

